@@ -2,10 +2,11 @@ package CGI::Lingua;
 
 use warnings;
 use strict;
-use Class::Autouse qw{Carp Locale::Language Locale::Object::Country Locale::Object::DB I18N::AcceptLanguage I18N::LangTags::Detect Storable};
+use Storable; # RT117983
+use Class::Autouse qw{Carp Locale::Language Locale::Object::Country Locale::Object::DB I18N::AcceptLanguage I18N::LangTags::Detect};
 
 use vars qw($VERSION);
-our $VERSION = '0.55';
+our $VERSION = '0.56';
 
 =head1 NAME
 
@@ -13,7 +14,7 @@ CGI::Lingua - Create a multilingual web page
 
 =head1 VERSION
 
-Version 0.55
+Version 0.56
 
 =cut
 
@@ -69,7 +70,7 @@ For a list of primary-codes refer to ISO-639 (e.g. 'en' for English).
 For a list of country-codes refer to ISO-3166 (e.g. 'gb' for United Kingdom).
 
     # We support English, French, British and American English, in that order
-    my $l = CGI::Lingua(supported => [('en', 'fr', 'en-gb', 'en-us')]);
+    my $l = CGI::Lingua(supported => ['en', 'fr', 'en-gb', 'en-us']);
 
 Takes optional parameter cache, an object which is used to cache country
 lookups.
@@ -156,6 +157,7 @@ sub new {
 	}, $class;
 }
 
+# Some of the information takes a long time to work out, so cache what we can
 sub DESTROY {
 	if(defined($^V) && ($^V ge 'v5.14.0')) {
 		return if ${^GLOBAL_PHASE} eq 'DESTRUCT';	# >= 5.14.0 only
@@ -169,14 +171,15 @@ sub DESTROY {
 	my $cache = $self->{_cache};
 	return unless($cache);
 
-	my $logger = $self->{_logger};
-
 	my $key = "$ENV{REMOTE_ADDR}/";
 	if($ENV{'HTTP_ACCEPT_LANGUAGE'}) {
 		$key .= "$ENV{HTTP_ACCEPT_LANGUAGE}/";
 	}
 	$key .= join('/', @{$self->{_supported}});
 	return if($cache->get($key));
+
+	my $logger = $self->{_logger};
+
 	if($logger) {
 		$logger->trace("Storing self in cache as $key");
 	}
@@ -251,6 +254,8 @@ language() returns 'Unknown'.
     # sublanguage will also be undefined, which may seem strange, but it
     # ensures that sites behave sensibly.
 
+If the script is not being run in a CGI environment, perhaps to debug it, the
+locale is used via the LANG environment variable.
 =cut
 
 sub language {
@@ -306,6 +311,9 @@ language_code_alpha2() returns undef.
 sub language_code_alpha2 {
 	my $self = shift;
 
+	if($self->{_logger}) {
+		$self->{_logger}->trace('Entered language_code_alpha2');
+	}
 	unless($self->{_slanguage}) {
 		$self->_find_language();
 	}
@@ -363,6 +371,9 @@ sub requested_language {
 sub _find_language {
 	my $self = shift;
 
+	if($self->{_logger}) {
+		$self->{_logger}->trace('Entered _find_language');
+	}
 	$self->{_rlanguage} = 'Unknown';
 	$self->{_slanguage} = 'Unknown';
 
@@ -392,6 +403,7 @@ sub _find_language {
 				$l = $1;
 			}
 		}
+
 		if($l) {
 			if($self->{_logger}) {
 				$self->{_logger}->debug("l: $l");
@@ -589,6 +601,19 @@ sub _find_language {
 	# or the requested language(s) isn't/aren't supported so use the IP
 	# address for an alternative
 	my $country = $self->country();
+
+	if(!defined($country)) {
+		if(defined($ENV{'LANG'})) {
+			# Running the script locally, presumably to debug, so set the language
+			# from the Locale
+			if($ENV{'LANG'} =~ /^(..)_(..)/) {
+				$country = $2;	# Best guess
+			} elsif($ENV{'LANG'} =~ /^(..)$/) {
+				$country = $1;	# Wrong, but maybe something will drop out
+			}
+		}
+	}
+
 	if(defined($country)) {
 		if($self->{_logger}) {
 			$self->{_logger}->debug("country: $country");
@@ -714,7 +739,6 @@ sub _get_closest {
 			last;
 		}
 	}
-	return;
 }
 
 =head2 country
@@ -739,6 +763,9 @@ sub country {
 	# waste time going through again and no doubt returning undef
 	# again.
 	if($self->{_country}) {
+		if($self->{_logger}) {
+			$self->{_logger}->trace('quick return: ' . $self->{_country});
+		}
 		return $self->{_country};
 	}
 
@@ -853,6 +880,22 @@ sub country {
 		delete($self->{_country});
 	}
 	unless($self->{_country}) {
+		if($self->{_logger}) {
+			$self->{_logger}->debug("Look up $ip on geoplugin");
+		}
+
+		if(eval { require LWP::Simple; require JSON::Parse } ) {
+			LWP::Simple->import();
+			JSON::Parse->import();
+
+			my $data = JSON::Parse::parse_json(LWP::Simple::get("http://www.geoplugin.net/json.gp?ip=$ip"));
+			$self->{_country} = $data->{'geoplugin_countryCode'};
+		}
+	}
+	unless($self->{_country}) {
+		if($self->{_logger}) {
+			$self->{_logger}->debug("Look up $ip on Whois");
+		}
 		require Net::Whois::IP;
 		Net::Whois::IP->import();
 
@@ -877,6 +920,10 @@ sub country {
 		}
 
 		unless($self->{_country}) {
+			if($self->{_logger}) {
+				$self->{_logger}->debug("Look up $ip on IANA");
+			}
+
 			require Net::Whois::IANA;
 			Net::Whois::IANA->import();
 
@@ -886,8 +933,12 @@ sub country {
 			};
 			unless ($@) {
 				$self->{_country} = $iana->country();
+				if($self->{_logger}) {
+					$self->{_logger}->debug("IANA reports $ip as " . $self->{_country});
+				}
 			}
 		}
+
 		if($self->{_country}) {
 			# 190.24.1.122 has carriage return in its WHOIS record
 			$self->{_country} =~ s/[\r\n]//g;
@@ -1112,15 +1163,17 @@ Nigel Horne, C<< <njh at bandsman.co.uk> >>
 If HTTP_ACCEPT_LANGUAGE is 3 characters, e.g., es-419,
 sublanguage() returns undef.
 
-Please report any bugs or feature requests to C<bug-cgi-lingua at rt.cpan.org>, or through
-the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=CGI-Lingua>.  I will be notified, and then you'll
+Please report any bugs or feature requests to C<bug-cgi-lingua at rt.cpan.org>,
+or through the web interface at
+L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=CGI-Lingua>.
+I will be notified, and then you'll
 automatically be notified of progress on your bug as I make changes.
 
 
 =head1 SEE ALSO
 
-Locale::Country::Object
-HTTP::BrowserDetect
+L<Locale::Country::Object>
+L<HTTP::BrowserDetect>
 
 =head1 SUPPORT
 
@@ -1157,7 +1210,7 @@ L<http://search.cpan.org/dist/CGI-Lingua/>
 
 =head1 LICENSE AND COPYRIGHT
 
-Copyright 2010-2015 Nigel Horne.
+Copyright 2010-2016 Nigel Horne.
 
 This program is released under the following licence: GPL
 
